
# Copyright 2021 Tecnativa - Carlos Roca
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl).

from odoo import http
from odoo.http import request
from odoo.addons.website.controllers.main import QueryURL

from odoo.addons.website_sale.controllers.main import WebsiteSale
from odoo.addons.website_sale.controllers.main import TableCompute
import logging


_logger = logging.getLogger(__name__)

class WebsiteSaleInheritController(WebsiteSale):
    @http.route('/shop/products/autocomplete', type='json', auth='public', website=True)
    def products_autocomplete(self, term, options={}, **kwargs):
        
        ProductTemplate = request.env['product.template']
        
        pricelist = request.env["website"].get_current_website().get_current_pricelist()
        
        display_description = options.get('display_description', True)
        display_price = options.get('display_price', True)
        order = self._get_search_order(options)
        max_nb_chars = options.get('max_nb_chars', 999)

        category = options.get('category')
        attrib_values = options.get('attrib_values')

        domain = self._get_search_domain(term, category, attrib_values, display_description)
        
        products = ProductTemplate.search(
            domain,
            limit=min(20, options.get('limit', 5)),
            order=order
        )

        fields = ['id', 'name', 'website_url']
        if display_description:
            fields.append('description_sale')

        res = {
            'products': products.read(fields),
            'products_count': ProductTemplate.search_count(domain),
        }

        if display_description:
            for res_product in res['products']:
                desc = res_product['description_sale']
                if desc and len(desc) > max_nb_chars:
                    res_product['description_sale'] = "%s..." % desc[:(max_nb_chars - 3)]

        if display_price:
            from_currency = request.env.user.company_id.currency_id
            to_currency = pricelist.currency_id
            for product in products:
                product_context = dict(request.env.context, quantity=1, pricelist=pricelist.id, partner=request.env.user.partner_id)
                product_id = product.with_context(product_context)
                res_product = next(p for p in res['products'] if p['id'] == product.id)
                res_product['price'] = request.env['ir.qweb.field.monetary'].value_to_html(product_id.price, {'from_currency': from_currency, 'to_currency': to_currency})

        return res

    @http.route(
        ["/sale/get_combination_info_pricelist_atributes"],
        type="json",
        auth="public",
        website=True,
    )
    def get_combination_info_pricelist_atributes(self, product_id, **kwargs):
        product = request.env["product.product"].browse(product_id)
        pricelist = request.env["website"].get_current_website().get_current_pricelist()

        qty_list = request.env["product.pricelist.item"].search(
            [
                "|",
                ("product_id", "=", product.id),
                "|",
                ("product_tmpl_id", "=", product.product_tmpl_id.id),
                (
                    "categ_id",
                    "in",
                    list(map(int, product.categ_id.parent_path.split("/")[0:-1])),
                ),
                ("min_quantity", ">", 0),
            ]
        )
        qty_list = sorted(set(qty_list.mapped("min_quantity")))
        res = []
        ctx = dict(request.env.context, pricelist=pricelist.id, quantity=0)
        last_price = product.with_context(ctx).price
        for min_qty in qty_list:
            ctx["quantity"] = min_qty
            new_price = product.with_context(ctx).price
            if new_price != last_price:
                res.append(
                    {
                        "min_qty": min_qty,
                        "price": new_price,
                        "currency": {
                            "position": product.currency_id.position,
                            "symbol": product.currency_id.symbol,
                        },
                    }
                )
                last_price = new_price
        return (res, product.uom_name)
